import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Representa una mesa de juego sobre la que hay una baraja de naipes
 */
public class Mesa extends World
{
    /** Los 4 tipos de palos de una baraja de naipes francesa. Utilizados para crear la baraja **/
    private String palos[] = {"diamantes", "corazones", "picas", "treboles"};

    /** Los 13 valores que tienen las cartas de un palo en la baraja francesa. Utilizados para crear la baraja */
    private String valores[] = {"A", "2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K"};
    
    /** La baraja de 52 cartas. Inicialmente se crean en orden pero se pueden desordenar en este arreglo. **/
    private Carta baraja[] = new Carta[palos.length * valores.length];
    
    private Carta cartaRevelada1;
    private Carta cartaRevelada2;
    
    private int espera = -1;
    
    /**
     * Construye la mesa de juego, la baraja desordenada y los marcadores
     */
    public Mesa()
    {
        super(960, 460, 1);
        
        crearFondo();
        crearBaraja();
        desordenar();
        mostrarBaraja();
    }
    
    /**
     * El fondo del escenario es una mesa, simulando que tiene una alfombra verde
     */
    private void crearFondo()
    {
        GreenfootImage fondo = getBackground();
        fondo.setColor( new java.awt.Color(0, 100, 0) );
        fondo.fillRect(0, 0, getWidth(), getHeight() );
    }
    
    /**
     * Crea cada una de las 52 cartas que conforman la baraja. Estas apareceran en orden en el
     * arreglo, primero por palo y luego por valor. Esto solo ocurre en la memoria de la
     * computadora y no tiene efecto visual en la pantalla
     */
    private void crearBaraja()
    {
        int indice = 0;
        for ( int palo = 0; palo < palos.length; ++palo )
            for ( int valor = 0; valor < valores.length; ++valor )
                baraja[indice++] = new Carta(palos[palo], valores[valor]);
    }
    
    /**
     * Desordena las cartas en la baraja. Esto solo ocurre en la memoria de la
     * computadora y no tiene efecto visual en la pantalla
     */
    public void desordenar()
    {
        for ( int i = 0; i < baraja.length; ++i )
        {
            int otra = Greenfoot.getRandomNumber(baraja.length);
            if ( i != otra )
            {
                // intercambiar la carta en la posicion i con la carta en la posicion otra
                Carta temporal = baraja[i];
                baraja[i] = baraja[otra];
                baraja[otra] = temporal;
            }
        }
    }
    
    /**
     * Muestra cada carta de la baraja en la mesa, de tal forma que todas se distribuyan en un
     * mosaico o matriz
     */
    private void mostrarBaraja()
    {
        int indice = 0;
        for ( int palo = 0; palo < palos.length; ++palo )
            for ( int valor = 0; valor < valores.length; ++valor )
                addObject( baraja[indice++], 10 + 36 + valor * 72, 50 + 48 + palo * 97 );
    }
    
    /**
     * 
     */
    public void act()
    {
        if ( espera > 0 )
            --espera;
            
        if ( espera == 0 )
        {
            cartaRevelada1.ocultar();
            cartaRevelada2.ocultar();
            cartaRevelada1 = cartaRevelada2 = null;
            espera = -1;
        }
    }
    
    public boolean puedoRevelarme(Carta cartaX)
    {
        if(cartaRevelada1 == null)
        {
            cartaRevelada1 = cartaX;
            return true;
        }
        else if (cartaRevelada2 == null && cartaX != cartaRevelada1)
        {
            cartaRevelada2 = cartaX;
            checkearPorPareja();
            return true;
        } else 
            return false;
    }
    
    public void checkearPorPareja()
    {
        if ( cartaRevelada1.valor().equals( cartaRevelada2.valor() ) )
            {
                removeObject( cartaRevelada1 );
                removeObject( cartaRevelada2 );
                cartaRevelada1 = cartaRevelada2 = null;
            }
        else
        {
            espera = 100;
        }
    }
}
